from __future__ import annotations

from typing import TYPE_CHECKING

from loguru import logger

from discord_reminder_bot.parsers import calculate, parse_time
from discord_reminder_bot.settings import scheduler

if TYPE_CHECKING:
    import datetime

    from apscheduler.job import Job


def add_reminder_job(
    message: str,
    time: str,
    channel_id: int,
    author_id: int,
    user_id: int | None = None,
    guild_id: int | None = None,
    dm_and_current_channel: bool | None = None,
) -> str:
    """Adds a reminder job to the scheduler based on user input.

    Schedules a message to be sent at a specified time either to a specific channel,
    a specific user via direct message, or both. It handles permission checks,
    time parsing, and job creation using the APScheduler instance.

    Args:
        message: The content of the reminder message to be sent.
        time: A string representing the date and time for the reminder.
            This string will be parsed to a datetime object.
        channel_id: The ID of the channel where the reminder will be sent.
        user_id: The Discord ID of the user to send a DM to. If None, no DM is sent.
        guild_id: The ID of the guild (server) where the reminder is set.
        author_id: The ID of the user who created the reminder.
        dm_and_current_channel: If True and a user is specified, sends the
            reminder to both the user's DM and the target channel. If False
            and a user is specified, only sends the DM. Defaults to None,
            behaving like False if only a user is specified, or sending only
            to the channel if no user is specified.

    Returns:
        The response message indicating the status of the reminder job creation.
    """
    dm_message: str = ""
    if user_id:
        parsed_time: datetime.datetime | None = parse_time(date_to_parse=time)
        if not parsed_time:
            return f"Failed to parse time: {time}."

        user_reminder: Job = scheduler.add_job(
            func="discord_reminder_bot.main:send_to_user",
            trigger="date",
            run_date=parsed_time,
            kwargs={
                "user_id": user_id,
                "guild_id": guild_id,
                "message": message,
            },
        )
        logger.info(f"User reminder job created: {user_reminder} for {user_id} at {parsed_time}")

        dm_message = f" and a DM to <@{user_id}>"
        if not dm_and_current_channel:
            return (
                f"Hello <@{author_id}>,\n"
                f"I will send a DM to <@{user_id}> at:\n"
                f"First run in {calculate(user_reminder)} with the message:\n**{message}**."
            )

    # Create channel reminder job
    channel_job: Job = scheduler.add_job(
        func="discord_reminder_bot.main:send_to_discord",
        trigger="date",
        run_date=parse_time(date_to_parse=time),
        kwargs={
            "channel_id": channel_id,
            "message": message,
            "author_id": author_id,
        },
    )
    logger.info(f"Channel reminder job created: {channel_job} for {channel_id}")

    return (
        f"Hello <@{author_id}>,\n"
        f"I will notify you in <#{channel_id}>{dm_message}.\n"
        f"First run in {calculate(channel_job)} with the message:\n**{message}**."
    )
