from __future__ import annotations

import datetime
import json
import tempfile
from typing import TYPE_CHECKING

import discord
from loguru import logger

if TYPE_CHECKING:
    from apscheduler.schedulers.asyncio import AsyncIOScheduler
    from discord.interactions import InteractionChannel


async def backup_reminder_job(interaction: discord.Interaction, scheduler: AsyncIOScheduler, all_servers: bool) -> None:
    """Backs up reminder jobs from the scheduler to a JSON file.

    Exports jobs from the provided AsyncIOScheduler to a temporary JSON file.
    If `all_servers` is False, it filters the jobs to include only those
    associated with the guild where the command was invoked. This requires
    the invoking user to have administrator permissions in that guild.
    The resulting list of jobs (either all or filtered) is then written
    to another temporary JSON file and sent as an attachment via the
    interaction followup, along with a confirmation message.

    Args:
        interaction: The discord interaction object that triggered the command.
        scheduler: The AsyncIOScheduler instance containing the jobs to back up.
        all_servers: If True, backs up jobs from all servers managed by the
            scheduler. If False, backs up only jobs associated with the
            server (guild) where the command was invoked.
    """
    logger.info(f"Backing up reminders for {interaction.user} ({interaction.user.id}) in {interaction.channel}")
    logger.info(f"Arguments: {locals()}")

    # Retrieve all jobs
    with tempfile.NamedTemporaryFile(mode="r+", delete=False, encoding="utf-8", suffix=".json") as temp_file:
        # Export jobs to a temporary file
        scheduler.export_jobs(temp_file.name)

        # Load the exported jobs
        temp_file.seek(0)
        jobs_data = json.load(temp_file)

    # Amount of jobs before filtering
    amount_of_jobs: int = len(jobs_data.get("jobs", []))

    if not all_servers:
        interaction_channel: InteractionChannel | None = interaction.channel
        if not interaction_channel:
            await interaction.followup.send(content="Failed to get channel.", ephemeral=True)
            return

        if not isinstance(interaction.user, discord.Member):
            await interaction.followup.send(content="Failed to get user.", ephemeral=True)
            return

        is_admin: bool = interaction_channel.permissions_for(interaction.user).administrator
        if not is_admin:
            await interaction.followup.send(content="You must be an administrator to backup all servers.", ephemeral=True)
            return

        # Can't be 0 because that's the default value for jobs without a guild
        guild_id: int = interaction.guild.id if interaction.guild else -1
        channels_in_this_guild: list[int] = [c.id for c in interaction.guild.channels] if interaction.guild else []
        logger.debug(f"Guild ID: {guild_id}")

        for job in jobs_data.get("jobs", []):
            # Check if the job is in the current guild
            job_guild_id = int(job.get("kwargs", {}).get("guild_id", 0))
            if job_guild_id and job_guild_id != guild_id:
                logger.debug(f"Skipping job: {job.get('id')} because it's not in the current guild.")
                jobs_data["jobs"].remove(job)

            # Check if the channel is in the current guild
            if job.get("kwargs", {}).get("channel_id") not in channels_in_this_guild:
                logger.debug(f"Skipping job: {job.get('id')} because it's not in the current guild.")
                jobs_data["jobs"].remove(job)

    msg: str = "All reminders in this server have been backed up." if not all_servers else "All reminders have been backed up."
    msg += "\nYou can restore them using `/remind restore`."

    if not all_servers:
        msg += f"\nAmount of jobs on all servers: {amount_of_jobs}, in this server: {len(jobs_data.get('jobs', []))}"
        msg += "\nYou can use `/remind backup all_servers:True` to backup all servers."
    else:
        msg += f"\nAmount of jobs: {amount_of_jobs}"

    # Write the data to a new file
    with tempfile.NamedTemporaryFile(mode="w", delete=False, encoding="utf-8", suffix=".json") as output_file:
        file_name: str = f"reminders-backup-{datetime.datetime.now(tz=scheduler.timezone)}.json"
        json.dump(jobs_data, output_file, indent=4)
        output_file.seek(0)

        await interaction.followup.send(content=msg, file=discord.File(output_file.name, filename=file_name))
