from __future__ import annotations

import logging
from typing import TYPE_CHECKING

import discord

from discord_reminder_bot.parsers import calculate

if TYPE_CHECKING:
    import discord
    from apscheduler.job import Job
    from apscheduler.schedulers.asyncio import AsyncIOScheduler

logger: logging.Logger = logging.getLogger("discord_reminder_bot")


async def cron_reminder_job(
    interaction: discord.Interaction,
    scheduler: AsyncIOScheduler,
    message: str,
    year: str | None = None,
    month: str | None = None,
    day: str | None = None,
    week: str | None = None,
    day_of_week: str | None = None,
    hour: str | None = None,
    minute: str | None = None,
    second: str | None = None,
    start_date: str | None = None,
    end_date: str | None = None,
    timezone: str | None = None,
    jitter: int | None = None,
    channel: discord.TextChannel | None = None,
    user: discord.User | None = None,
    dm_and_current_channel: bool | None = None,
) -> None:
    """Create a new cron job.

    Args that are None will be defaulted to *.

    Args:
        interaction (discord.Interaction): The interaction object for the command.
        scheduler (AsyncIOScheduler): The scheduler to add the job to.
        message (str): The content of the reminder.
        year (str): 4-digit year. Defaults to *.
        month (str): Month (1-12). Defaults to *.
        day (str): Day of the month (1-31). Defaults to *.
        week (str): ISO Week of the year (1-53). Defaults to *.
        day_of_week (str): Number or name of weekday (0-6 or mon,tue,wed,thu,fri,sat,sun).
        hour (str): Hour (0-23). Defaults to *.
        minute (str): Minute (0-59). Defaults to *.
        second (str): Second (0-59). Defaults to *.
        start_date (str): Earliest possible date/time to trigger on (inclusive). Will get parsed.
        end_date (str): Latest possible date/time to trigger on (inclusive). Will get parsed.
        timezone (str): Time zone to use for the date/time calculations Defaults to scheduler timezone.
        jitter (int, optional): Delay the job execution by jitter seconds at most.
        channel (discord.TextChannel, optional): The channel to send the reminder to. Defaults to current channel.
        user (discord.User, optional): Send reminder as a DM to this user. Defaults to None.
        dm_and_current_channel (bool, optional): If user is provided, send reminder as a DM to the user and in this channel. Defaults to only the user.
    """
    # Log kwargs
    logger.info("New cron job from %s (%s) in %s", interaction.user, interaction.user.id, interaction.channel)
    logger.info("Cron job arguments: %s", locals())

    # Get the channel ID
    channel_id: int | None = channel.id if channel else (interaction.channel.id if interaction.channel else None)
    if not channel_id:
        await interaction.followup.send(content="Failed to get channel.", ephemeral=True)
        return

    # Ensure the guild is valid
    guild: discord.Guild | None = interaction.guild or None
    if not guild:
        await interaction.followup.send(content="Failed to get guild.", ephemeral=True)
        return

    # Create user DM reminder job if user is specified
    dm_message: str = ""
    if user:
        user_reminder: Job = scheduler.add_job(
            func="discord_reminder_bot.main:send_to_user",
            trigger="cron",
            year=year,
            month=month,
            day=day,
            week=week,
            day_of_week=day_of_week,
            hour=hour,
            minute=minute,
            second=second,
            start_date=start_date,
            end_date=end_date,
            timezone=timezone,
            jitter=jitter,
            kwargs={
                "user_id": user.id,
                "guild_id": guild.id,
                "message": message,
            },
        )

        dm_message = f" and a DM to {user.display_name}"
        if not dm_and_current_channel:
            await interaction.followup.send(
                content=f"Hello {interaction.user.display_name},\n"
                f"I will send a DM to {user.display_name} at:\n"
                f"First run in {calculate(user_reminder)} with the message:\n**{message}**.",
            )
            return

    # Create channel reminder job
    channel_job: Job = scheduler.add_job(
        func="discord_reminder_bot.main:send_to_discord",
        trigger="cron",
        year=year,
        month=month,
        day=day,
        week=week,
        day_of_week=day_of_week,
        hour=hour,
        minute=minute,
        second=second,
        start_date=start_date,
        end_date=end_date,
        timezone=timezone,
        jitter=jitter,
        kwargs={
            "channel_id": channel_id,
            "message": message,
            "author_id": interaction.user.id,
        },
    )

    await interaction.followup.send(
        content=f"Hello {interaction.user.display_name},\n"
        f"I will notify you in <#{channel_id}>{dm_message}.\n"
        f"First run in {calculate(channel_job)} with the message:\n**{message}**.",
    )
