from __future__ import annotations

import datetime

import discord
from loguru import logger

from discord_reminder_bot.parsers import parse_time


async def add_discord_event(
    interaction: discord.Interaction,
    message: str,
    event_start: str,
    event_end: str,
    location: str,
    reason: str | None = None,
) -> None:
    """Creates a new Discord scheduled event based on user input.

    This command handles the creation of a scheduled event within the Discord guild
    where the interaction originated. It parses the provided start and end times,
    validates permissions, and creates the event using the Discord API. If the
    specified start time is in the past, it adjusts the start time to be slightly
    in the future and starts the event immediately.

    Args:
        interaction: The discord interaction object representing the command invocation.
        message: The name or description for the scheduled event.
        event_start: A string representing the desired start time for the event.
                     This string will be parsed into a datetime object.
        event_end: A string representing the desired end time for the event.
                   This string will be parsed into a datetime object.
        location: A string specifying the location for the event (e.g., a URL or physical address).
        reason: An optional string providing a reason for the event creation,
                visible in the guild's audit log. Defaults to a standard message
                indicating the creator if None.
    """
    logger.info(f"New event from {interaction.user} ({interaction.user.id}) in {interaction.channel}")
    logger.info(f"Arguments: {locals()}")

    # Check if we have a valid guild
    guild: discord.Guild | None = interaction.guild
    if not guild:
        await interaction.followup.send(content="This command can only be used in a server.", ephemeral=True)
        return

    # Check if we have permission to create events
    if not guild.me.guild_permissions.create_events:
        await interaction.followup.send(content="I don't have permission to create events in this guild.", ephemeral=True)
        return

    event_start_time: datetime.datetime | None = parse_time(date_to_parse=event_start)
    event_end_time: datetime.datetime | None = parse_time(date_to_parse=event_end)

    if not event_start_time or not event_end_time:
        await interaction.followup.send(content=f"Failed to parse time: {event_start} or {event_end}.", ephemeral=True)
        return

    # If event_start_time is in the past, make it now + 5 seconds
    start_immediately: bool = False
    if event_start_time < datetime.datetime.now(event_start_time.tzinfo):
        start_immediately = True
        event_start_time = datetime.datetime.now(event_start_time.tzinfo) + datetime.timedelta(seconds=5)
        await interaction.followup.send(content="Event start time was in the past. Starting event in 5 seconds instead.")

    reason_msg: str = f"Event created by {interaction.user} ({interaction.user.id})."

    event: discord.ScheduledEvent = await guild.create_scheduled_event(
        name=message,
        start_time=event_start_time,
        entity_type=discord.EntityType.external,
        privacy_level=discord.PrivacyLevel.guild_only,
        end_time=event_end_time,
        reason=reason or reason_msg,
        location=location,
    )

    if start_immediately:
        await event.start()

    msg: str = f"Event '{event.name}' created successfully!\n"

    if event.start_time:
        msg += f"Start Time: <t:{int(event.start_time.timestamp())}:R>\n"

    if event.end_time:
        msg += f"End Time: <t:{int(event.end_time.timestamp())}:R>\n"

    if event.channel_id:
        msg += f"Channel: <#{event.channel_id}>\n"

    if event.location:
        msg += f"Location: {event.location}\n"

    if event.creator_id:
        msg += f"Created by: <@{event.creator_id}>"

    await interaction.followup.send(content=msg)
