from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from discord_reminder_bot.parsers import calculate
from discord_reminder_bot.settings import scheduler

if TYPE_CHECKING:
    import discord
    from apscheduler.job import Job
    from discord.interactions import InteractionChannel


logger: logging.Logger = logging.getLogger("discord_reminder_bot")


async def interval_reminder_job(
    interaction: discord.Interaction,
    message: str,
    weeks: int = 0,
    days: int = 0,
    hours: int = 0,
    minutes: int = 0,
    seconds: int = 0,
    start_date: str | None = None,
    end_date: str | None = None,
    timezone: str | None = None,
    jitter: int | None = None,
    channel: discord.TextChannel | None = None,
    user: discord.User | None = None,
    dm_and_current_channel: bool | None = None,
) -> None:
    """Create a new reminder that triggers based on an interval.

    Args:
        interaction (discord.Interaction): The interaction object for the command.
        message (str): The content of the reminder.
        weeks (int, optional): Number of weeks between each run. Defaults to 0.
        days (int, optional): Number of days between each run. Defaults to 0.
        hours (int, optional): Number of hours between each run. Defaults to 0.
        minutes (int, optional): Number of minutes between each run. Defaults to 0.
        seconds (int, optional): Number of seconds between each run. Defaults to 0.
        start_date (str, optional): Earliest possible date/time to trigger on (inclusive). Will get parsed.
        end_date (str, optional): Latest possible date/time to trigger on (inclusive). Will get parsed.
        timezone (str, optional): Time zone to use for the date/time calculations Defaults to scheduler timezone.
        jitter (int, optional): Delay the job execution by jitter seconds at most.
        channel (discord.TextChannel, optional): The channel to send the reminder to. Defaults to current channel.
        user (discord.User, optional): Send reminder as a DM to this user. Defaults to None.
        dm_and_current_channel (bool, optional): If user is provided, send reminder as a DM to the user and in this channel. Defaults to only the user.
    """
    logger.info("New interval job from %s (%s) in %s", interaction.user, interaction.user.id, interaction.channel)
    logger.info("Arguments: %s", locals())

    # Only allow intervals of 30 seconds or more so we don't spam the channel
    if weeks == days == hours == minutes == 0 and seconds < 30:
        await interaction.followup.send(content="Interval must be at least 30 seconds.", ephemeral=True)
        return

    # Check if we have access to the specified channel or the current channel
    target_channel: InteractionChannel | None = channel or interaction.channel
    if target_channel and interaction.guild and not target_channel.permissions_for(interaction.guild.me).send_messages:
        await interaction.followup.send(
            content=f"I don't have permission to send messages in <#{target_channel.id}>.",
            ephemeral=True,
        )

    # Get the channel ID
    channel_id: int | None = channel.id if channel else (interaction.channel.id if interaction.channel else None)
    if not channel_id:
        await interaction.followup.send(content="Failed to get channel.", ephemeral=True)
        return

    # Ensure the guild is valid
    guild: discord.Guild | None = interaction.guild or None
    if not guild:
        await interaction.followup.send(content="Failed to get guild.", ephemeral=True)
        return

    # Create user DM reminder job if user is specified
    dm_message: str = ""
    if user:
        dm_job: Job = scheduler.add_job(
            func="discord_reminder_bot.main:send_to_user",
            trigger="interval",
            weeks=weeks,
            days=days,
            hours=hours,
            minutes=minutes,
            seconds=seconds,
            start_date=start_date,
            end_date=end_date,
            timezone=timezone,
            jitter=jitter,
            kwargs={
                "user_id": user.id,
                "guild_id": guild.id,
                "message": message,
            },
        )

        dm_message = f" and a DM to {user.display_name} "
        if not dm_and_current_channel:
            await interaction.followup.send(
                content=f"Hello {interaction.user.display_name},\n"
                f"I will send a DM to {user.display_name} at:\n"
                f"First run in {calculate(dm_job)} with the message:\n**{message}**.",
            )

    # Create channel reminder job
    # TODO(TheLovinator): Test that "discord_reminder_bot.main:send_to_discord" is always there  # noqa: TD003
    channel_job: Job = scheduler.add_job(
        func="discord_reminder_bot.main:send_to_discord",
        trigger="interval",
        weeks=weeks,
        days=days,
        hours=hours,
        minutes=minutes,
        seconds=seconds,
        start_date=start_date,
        end_date=end_date,
        timezone=timezone,
        jitter=jitter,
        kwargs={
            "channel_id": channel_id,
            "message": message,
            "author_id": interaction.user.id,
        },
    )

    await interaction.followup.send(
        content=f"Hello {interaction.user.display_name},\n"
        f"I will notify you in <#{channel_id}>{dm_message}.\n"
        f"First run in {calculate(channel_job)} with the message:\n**{message}**.",
    )
