from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from apscheduler.jobstores.base import JobLookupError

if TYPE_CHECKING:
    import discord
    from apscheduler.job import Job
    from apscheduler.schedulers.asyncio import AsyncIOScheduler

logger: logging.Logger = logging.getLogger("discord_reminder_bot")


async def pause_reminder_job(interaction: discord.Interaction, job_id: str, scheduler: AsyncIOScheduler) -> None:
    """Pauses a specific reminder job in the APScheduler.

    This function attempts to find and pause a job identified by `job_id`
    within the provided `scheduler`. It logs the action and sends feedback
    to the user via the Discord interaction, indicating success or failure.

    Args:
        interaction: The Discord interaction object representing the command invocation.
        job_id: The unique identifier of the reminder job to pause.
        scheduler: The AsyncIOScheduler instance managing the reminder jobs.
    """
    logger.info("Pausing reminder with ID %s for %s (%s) in %s", job_id, interaction.user, interaction.user.id, interaction.channel)
    logger.debug("Arguments: %s", locals())

    logger.debug("Attempting to pause job...")
    try:
        job: Job | None = scheduler.get_job(job_id)
        if not job:
            await interaction.followup.send(content=f"Reminder with ID {job_id} not found.", ephemeral=True)
            return
        scheduler.pause_job(job_id)
        logger.info("Paused job %s.", job_id)
        await interaction.followup.send(content=f"Reminder with ID {job_id} paused successfully.")
    except JobLookupError as e:
        logger.exception("Failed to pause job %s", job_id)
        await interaction.followup.send(content=f"Failed to pause reminder with ID {job_id}. {e}", ephemeral=True)

    logger.info("Job %s paused in the scheduler.", job_id)
