from __future__ import annotations

from typing import TYPE_CHECKING

from apscheduler.jobstores.base import JobLookupError
from loguru import logger

from discord_reminder_bot.markdown_utils import generate_markdown_state

if TYPE_CHECKING:
    import discord
    from apscheduler.job import Job
    from apscheduler.schedulers.asyncio import AsyncIOScheduler


async def remove_reminder_job(interaction: discord.Interaction, job_id: str, scheduler: AsyncIOScheduler) -> None:
    """Removes a specific reminder job from the scheduler based on its ID.

    This function attempts to find a job associated with the given `job_id`
    within the provided `scheduler`. If the job exists, it is removed,
    and a confirmation message detailing the removed job's state is sent
    as a follow-up to the interaction. If the job cannot be found, or if
    an error occurs during the removal process (like `JobLookupError`),
    an appropriate error message is sent as an ephemeral follow-up.

    Args:
        interaction (discord.Interaction): The Discord interaction object
            that triggered the command. Used for sending follow-up messages.
        job_id (str): The unique identifier of the reminder job to be removed.
        scheduler (AsyncIOScheduler): The APScheduler instance managing the
            scheduled reminder jobs.

    """
    logger.debug(f"Removing reminder with ID {job_id} for {interaction.user} ({interaction.user.id}) in {interaction.channel}")
    logger.debug(f"Arguments: {locals()}")

    try:
        job: Job | None = scheduler.get_job(job_id)
        if not job:
            await interaction.followup.send(content=f"Reminder with ID {job_id} not found.", ephemeral=True)
            return
        scheduler.remove_job(job_id)
        logger.info(f"Removed job {job_id}. {job.__getstate__()}")
        await interaction.followup.send(
            content=f"Reminder with ID {job_id} removed successfully.\n{generate_markdown_state(job.__getstate__())}",
        )
    except JobLookupError as e:
        logger.exception(f"Failed to remove job {job_id}")
        await interaction.followup.send(content=f"Failed to remove reminder with ID {job_id}. {e}", ephemeral=True)

    logger.info(f"Job {job_id} removed from the scheduler.")
