from __future__ import annotations

import json
import logging
import tempfile
from pathlib import Path
from typing import TYPE_CHECKING

import discord

from discord_reminder_bot.parsers import calculate

if TYPE_CHECKING:
    import discord
    from apscheduler.job import Job
    from apscheduler.schedulers.asyncio import AsyncIOScheduler

logger: logging.Logger = logging.getLogger("discord_reminder_bot")


async def restore_reminder_job(bot: discord.Client, interaction: discord.Interaction, scheduler: AsyncIOScheduler) -> None:  # noqa: PLR0915
    """Restores reminder jobs from a user-provided JSON backup file.

    Prompts the user via the interaction followup to reply with a JSON file
    containing the backup data. Waits for a message reply from the same user
    in the same channel containing a single JSON attachment.
    Validates the reply and attachment. Downloads the JSON file to a temporary
    location, loads the job data, filters out any jobs that already exist in the
    scheduler (based on job ID), and imports the remaining jobs using
    `scheduler.import_jobs`.
    Finally, sends a confirmation message listing the newly added jobs or
    indicates if no new jobs were added.

    Args:
        bot (discord.Client): The Discord bot client instance, used to wait for
            the user's reply message.
        interaction (discord.Interaction): The interaction object representing the
            command invocation. Used for sending messages back to the user.
        scheduler (AsyncIOScheduler): The APScheduler instance where the jobs
            will be restored.
    """
    logger.info("Restoring reminders from file for %s (%s) in %s", interaction.user, interaction.user.id, interaction.channel)
    logger.info("Arguments: %s", locals())

    # Tell to reply with the file to this message
    await interaction.followup.send(content="Please reply to this message with the backup file.")

    # Get the old jobs
    old_jobs: list[Job] = scheduler.get_jobs()

    # Wait for the reply
    while True:
        try:
            reply: discord.Message | None = await bot.wait_for("message", timeout=60, check=lambda m: m.author == interaction.user)
        except TimeoutError:
            edit_msg = "~~Please reply to this message with the backup file.~~\nTimed out after 60 seconds."
            await interaction.edit_original_response(content=edit_msg)
            return

        if not reply:
            await interaction.followup.send(content="No reply found. Please try again.")
            continue

        if not reply.channel:
            await interaction.followup.send(content="No channel found. Please try again.")
            continue

        # Fetch the message by its ID to ensure we have the latest data
        reply = await reply.channel.fetch_message(reply.id)

        if not reply or not reply.attachments:
            await interaction.followup.send(content="No file attached. Please try again.")
            continue
        break

    # Get the attachment
    attachment: discord.Attachment = reply.attachments[0]
    if not attachment or not attachment.filename.endswith(".json"):
        await interaction.followup.send(
            content=f"Invalid file type. Should be a JSON file not '{attachment.filename}'. Please try again.",
        )
        return

    jobs_already_exist: list[str] = []

    # Save the file to a temporary file and import the jobs
    with tempfile.NamedTemporaryFile(mode="w+", delete=False, encoding="utf-8", suffix=".json") as temp_file:
        logger.info("Saving attachment to %s", temp_file.name)
        await attachment.save(Path(temp_file.name))

        # Load the jobs data from the file
        temp_file.seek(0)
        jobs_data: dict = json.load(temp_file)

        logger.info("Importing jobs from file")
        logger.debug("Jobs data: %s", jobs_data)

        with tempfile.NamedTemporaryFile(mode="w+", delete=False, encoding="utf-8", suffix=".json") as temp_import_file:
            # We can't import jobs with the same ID so remove them from the JSON
            jobs = [job for job in jobs_data.get("jobs", []) if not scheduler.get_job(job.get("id"))]
            jobs_already_exist = [job.get("id") for job in jobs_data.get("jobs", []) if scheduler.get_job(job.get("id"))]
            jobs_data["jobs"] = jobs
            for job_id in jobs_already_exist:
                logger.debug("Skipping importing '%s' because it already exists in the db.", job_id)

            logger.debug("Jobs data after filtering: %s", jobs_data)
            logger.info("Jobs already exist: %s", jobs_already_exist)

            # Write the new data to a temporary file
            json.dump(jobs_data, temp_import_file)
            temp_import_file.seek(0)

            # Import the jobs
            scheduler.import_jobs(temp_import_file.name)

    # Get the new jobs
    new_jobs: list[Job] = scheduler.get_jobs()

    # Get the difference
    added_jobs: list[Job] = [job for job in new_jobs if job not in old_jobs]

    if added_jobs:
        msg: str = "Reminders restored successfully.\nAdded jobs:\n"
        for j in added_jobs:
            msg += f"* Message: **{j.kwargs.get('message', 'No message found')}** {calculate(j) or 'N/A'}\n"

        await interaction.followup.send(content=msg)
    else:
        await interaction.followup.send(content="No new reminders were added.")
