from __future__ import annotations

from typing import TYPE_CHECKING

from apscheduler.jobstores.base import JobLookupError
from loguru import logger

if TYPE_CHECKING:
    import discord
    from apscheduler.job import Job
    from apscheduler.schedulers.asyncio import AsyncIOScheduler


async def unpause_reminder(interaction: discord.Interaction, job_id: str, scheduler: AsyncIOScheduler) -> None:
    """Unpauses a scheduled reminder job.

    Finds a job by its ID in the scheduler and resumes it. Sends feedback
    to the user via the interaction about the success or failure of the operation.

    Args:
        interaction: The discord interaction object representing the command invocation.
        job_id: The unique identifier of the reminder job to unpause.
        scheduler: The scheduler instance to manage the job.
    """
    logger.debug(f"Unpausing reminder with ID {job_id} for {interaction.user} ({interaction.user.id}) in {interaction.channel}")
    logger.debug(f"Arguments: {locals()}")

    try:
        job: Job | None = scheduler.get_job(job_id)
        if not job:
            await interaction.followup.send(content=f"Reminder with ID {job_id} not found.", ephemeral=True)
            return
        scheduler.resume_job(job_id)
        logger.info(f"Unpaused job {job_id}.")
        await interaction.followup.send(content=f"Reminder with ID {job_id} unpaused successfully.")
    except JobLookupError as e:
        logger.exception(f"Failed to unpause job {job_id}")
        await interaction.followup.send(content=f"Failed to unpause reminder with ID {job_id}. {e}", ephemeral=True)

    logger.info(f"Job {job_id} unpaused in the scheduler.")
