from __future__ import annotations

import json
from typing import TYPE_CHECKING, Any

import discord
from apscheduler.job import Job
from apscheduler.triggers.date import DateTrigger
from apscheduler.triggers.interval import IntervalTrigger
from loguru import logger

from discord_reminder_bot.parsers import get_human_readable_time
from interactions.api.models.misc import Snowflake

if TYPE_CHECKING:
    from apscheduler.schedulers.asyncio import AsyncIOScheduler
    from discord.guild import GuildChannel
    from discord.types.channel import _BaseChannel


def generate_markdown_state(state: dict[str, Any]) -> str:
    """Format the __getstate__ dictionary for Discord markdown.

    Args:
        state (dict): The __getstate__ dictionary.

    Returns:
        str: The formatted string.
    """
    if not state:
        return "```json\nNo state found.\n```"

    # Convert the IntervalTrigger to a string representation
    for key, value in state.items():
        if isinstance(value, IntervalTrigger):
            state[key] = "IntervalTrigger"
        elif isinstance(value, DateTrigger):
            state[key] = "DateTrigger"
        elif isinstance(value, Job):
            state[key] = "Job"
        elif isinstance(value, Snowflake):
            state[key] = str(value)

    try:
        msg: str = json.dumps(state, indent=4, default=str)
    except TypeError as e:
        e.add_note("This is likely due to a non-serializable object in the state. Please check the state for any non-serializable objects.")
        e.add_note(f"{state=}")
        logger.error(f"Failed to serialize state: {e}")
        return "```json\nFailed to serialize state.\n```"

    return "```json\n" + msg + "\n```"


def generate_reminder_summary(ctx: discord.Interaction, bot: discord.Client, scheduler: AsyncIOScheduler) -> list[str]:  # noqa: PLR0912
    """Create a message with all the jobs, splitting messages into chunks of up to 2000 characters.

    Args:
        ctx (discord.Interaction): The context of the interaction.
        bot (discord.Client): The Discord bot client.
        scheduler (AsyncIOScheduler): The scheduler to get the jobs from.

    Returns:
        list[str]: A list of messages with all the jobs.
    """
    jobs: list[Job] = scheduler.get_jobs()
    msgs: list[str] = []

    guild: discord.Guild | None = None
    if isinstance(ctx.channel, discord.abc.GuildChannel):
        guild = ctx.channel.guild

    channels: list[GuildChannel] | list[_BaseChannel] = list(guild.channels) if guild else []
    channels_in_this_guild: list[int] = [c.id for c in channels]
    jobs_in_guild: list[Job] = []
    for job in jobs:
        guild_id: int = guild.id if guild else -1

        guild_id_from_kwargs = int(job.kwargs.get("guild_id", 0))

        if guild_id_from_kwargs and guild_id_from_kwargs != guild_id:
            logger.debug(f"Skipping job: {job.id} because it's not in the current guild.")
            continue

        if job.kwargs.get("channel_id") not in channels_in_this_guild:
            logger.debug(f"Skipping job: {job.id} because it's not in the current guild.")
            continue

        logger.debug(f"Adding job: {job.id} to the list.")
        jobs_in_guild.append(job)

    if len(jobs) != len(jobs_in_guild):
        logger.info(f"Filtered out {len(jobs) - len(jobs_in_guild)} jobs that are not in the current guild.")

    jobs = jobs_in_guild

    if not jobs:
        return ["No scheduled jobs found in the database."]

    header = (
        "You can use the following commands to manage reminders:\n"
        "Only jobs in the current guild are shown.\n"
        "`/remind pause <job_id>` - Pause a reminder\n"
        "`/remind unpause <job_id>` - Unpause a reminder\n"
        "`/remind remove <job_id>` - Remove a reminder\n"
        "`/remind modify <job_id>` - Modify the time of a reminder\n"
        "List of all reminders:\n"
    )

    current_msg: str = header

    for job in jobs:
        # Build job-specific message
        job_msg: str = "```md\n"
        job_msg += f"# {job.kwargs.get('message', '')}\n"
        job_msg += f"   * {job.id}\n"
        job_msg += f"   * {job.trigger} {get_human_readable_time(job)}"

        if job.kwargs.get("user_id"):
            job_msg += f" <@{job.kwargs.get('user_id')}>"
        if job.kwargs.get("channel_id"):
            channel = bot.get_channel(job.kwargs.get("channel_id"))
            if channel and isinstance(channel, discord.abc.GuildChannel | discord.Thread):
                job_msg += f" in #{channel.name}"

        if job.kwargs.get("guild_id"):
            guild = bot.get_guild(job.kwargs.get("guild_id"))
            if guild:
                job_msg += f" in {guild.name}"
            job_msg += f" {job.kwargs.get('guild_id')}"

        job_msg += "```"

        # If adding this job exceeds 2000 characters, push the current message and start a new one.
        if len(current_msg) + len(job_msg) > 2000:
            msgs.append(current_msg)
            current_msg = job_msg
        else:
            current_msg += job_msg

    # Append any remaining content in current_msg.
    if current_msg:
        msgs.append(current_msg)

    return msgs
